@@ -15,50 +15,43 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-import re
-from typing import TYPE_CHECKING, Any, Dict, Iterable, List, Mapping, Optional, Sequence, SupportsAbs, Union
+from __future__ import annotations
 
-from packaging.version import Version
+import ast
+import re
+from typing import TYPE_CHECKING, Any, Callable, Iterable, Mapping, NoReturn, Sequence, SupportsAbs
 
 from airflow.compat.functools import cached_property
-from airflow.exceptions import AirflowException
+from airflow.exceptions import AirflowException, AirflowFailException
 from airflow.hooks.base import BaseHook
 from airflow.models import BaseOperator, SkipMixin
-from airflow.providers.common.sql.hooks.sql import DbApiHook, _backported_get_hook
-from airflow.version import version
+from airflow.providers.common.sql.hooks.sql import DbApiHook, fetch_all_handler
 
 if TYPE_CHECKING:
     from airflow.utils.context import Context
 
 
-def parse_boolean(val: str) -> Union[str, bool]:
+def _convert_to_float_if_possible(s: str) -> float | str:
+    try:
+        return float(s)
+    except (ValueError, TypeError):
+        return s
+
+
+def _parse_boolean(val: str) -> str | bool:
     """Try to parse a string into boolean.
 
     Raises ValueError if the input is not a valid true- or false-like string value.
     """
     val = val.lower()
-    if val in ('y', 'yes', 't', 'true', 'on', '1'):
+    if val in ("y", "yes", "t", "true", "on", "1"):
         return True
-    if val in ('n', 'no', 'f', 'false', 'off', '0'):
+    if val in ("n", "no", "f", "false", "off", "0"):
         return False
     raise ValueError(f"{val!r} is not a boolean-like string value")
 
 
-def _get_failed_checks(checks, col=None):
-    if col:
-        return [
-            f"Column: {col}\nCheck: {check},\nCheck Values: {check_values}\n"
-            for check, check_values in checks.items()
-            if not check_values["success"]
-        ]
-    return [
-        f"\tCheck: {check},\n\tCheck Values: {check_values}\n"
-        for check, check_values in checks.items()
-        if not check_values["success"]
-    ]
-
-
-_PROVIDERS_MATCHER = re.compile(r'airflow\.providers\.(.*)\.hooks.*')
+_PROVIDERS_MATCHER = re.compile(r"airflow\.providers\.(.*)\.hooks.*")
 
 _MIN_SUPPORTED_PROVIDERS_VERSION = {
     "amazon": "4.1.0",
@@ -92,34 +85,32 @@ class BaseSQLOperator(BaseOperator):
 
     The provided method is .get_db_hook(). The default behavior will try to
     retrieve the DB hook based on connection type.
-    You can custom the behavior by overriding the .get_db_hook() method.
+    You can customize the behavior by overriding the .get_db_hook() method.
+
+    :param conn_id: reference to a specific database
     """
 
     def __init__(
         self,
         *,
-        conn_id: Optional[str] = None,
-        database: Optional[str] = None,
-        hook_params: Optional[Dict] = None,
+        conn_id: str | None = None,
+        database: str | None = None,
+        hook_params: dict | None = None,
+        retry_on_failure: bool = True,
         **kwargs,
     ):
         super().__init__(**kwargs)
         self.conn_id = conn_id
         self.database = database
         self.hook_params = {} if hook_params is None else hook_params
+        self.retry_on_failure = retry_on_failure
 
     @cached_property
     def _hook(self):
         """Get DB Hook based on connection type"""
         self.log.debug("Get connection for %s", self.conn_id)
         conn = BaseHook.get_connection(self.conn_id)
-        if Version(version) >= Version('2.3'):
-            # "hook_params" were introduced to into "get_hook()" only in Airflow 2.3.
-            hook = conn.get_hook(hook_params=self.hook_params)  # ignore airflow compat check
-        else:
-            # For supporting Airflow versions < 2.3, we backport "get_hook()" method. This should be removed
-            # when "apache-airflow-providers-common-sql" will depend on Airflow >= 2.3.
-            hook = _backported_get_hook(conn, hook_params=self.hook_params)
+        hook = conn.get_hook(hook_params=self.hook_params)
         if not isinstance(hook, DbApiHook):
             from airflow.hooks.dbapi_hook import DbApiHook as _DbApiHook
 
@@ -133,16 +124,16 @@ class BaseSQLOperator(BaseOperator):
                     min_version = _MIN_SUPPORTED_PROVIDERS_VERSION.get(provider)
                     if min_version:
                         raise AirflowException(
-                            f'You are trying to use common-sql with {hook.__class__.__name__},'
-                            f' but the Hook class comes from provider {provider} that does not support it.'
-                            f' Please upgrade provider {provider} to at least {min_version}.'
+                            f"You are trying to use common-sql with {hook.__class__.__name__},"
+                            f" but the Hook class comes from provider {provider} that does not support it."
+                            f" Please upgrade provider {provider} to at least {min_version}."
                         )
             raise AirflowException(
-                f'You are trying to use `common-sql` with {hook.__class__.__name__},'
-                ' but its provider does not support it. Please upgrade the provider to a version that'
-                ' supports `common-sql`. The hook class should be a subclass of'
-                ' `airflow.providers.common.sql.hooks.sql.DbApiHook`.'
-                f' Got {hook.__class__.__name__} Hook with class hierarchy: {hook.__class__.mro()}'
+                f"You are trying to use `common-sql` with {hook.__class__.__name__},"
+                " but its provider does not support it. Please upgrade the provider to a version that"
+                " supports `common-sql`. The hook class should be a subclass of"
+                " `airflow.providers.common.sql.hooks.sql.DbApiHook`."
+                f" Got {hook.__class__.__name__} Hook with class hierarchy: {hook.__class__.mro()}"
             )
 
         if self.database:
@@ -155,10 +146,86 @@ class BaseSQLOperator(BaseOperator):
         Get the database hook for the connection.
 
         :return: the database hook object.
-        :rtype: DbApiHook
         """
         return self._hook
 
+    def _raise_exception(self, exception_string: str) -> NoReturn:
+        if self.retry_on_failure:
+            raise AirflowException(exception_string)
+        raise AirflowFailException(exception_string)
+
+
+class SQLExecuteQueryOperator(BaseSQLOperator):
+    """
+    Executes SQL code in a specific database
+    :param sql: the SQL code or string pointing to a template file to be executed (templated).
+    File must have a '.sql' extensions.
+    :param autocommit: (optional) if True, each command is automatically committed (default: False).
+    :param parameters: (optional) the parameters to render the SQL query with.
+    :param handler: (optional) the function that will be applied to the cursor (default: fetch_all_handler).
+    :param split_statements: (optional) if split single SQL string into statements (default: False).
+    :param return_last: (optional) if return the result of only last statement (default: True).
+
+    .. seealso::
+        For more information on how to use this operator, take a look at the guide:
+        :ref:`howto/operator:SQLExecuteQueryOperator`
+    """
+
+    template_fields: Sequence[str] = ("sql", "parameters")
+    template_ext: Sequence[str] = (".sql", ".json")
+    template_fields_renderers = {"sql": "sql", "parameters": "json"}
+    ui_color = "#cdaaed"
+
+    def __init__(
+        self,
+        *,
+        sql: str | list[str],
+        autocommit: bool = False,
+        parameters: Mapping | Iterable | None = None,
+        handler: Callable[[Any], Any] = fetch_all_handler,
+        split_statements: bool = False,
+        return_last: bool = True,
+        **kwargs,
+    ) -> None:
+        super().__init__(**kwargs)
+        self.sql = sql
+        self.autocommit = autocommit
+        self.parameters = parameters
+        self.handler = handler
+        self.split_statements = split_statements
+        self.return_last = return_last
+
+    def execute(self, context):
+        self.log.info("Executing: %s", self.sql)
+        hook = self.get_db_hook()
+        if self.do_xcom_push:
+            output = hook.run(
+                sql=self.sql,
+                autocommit=self.autocommit,
+                parameters=self.parameters,
+                handler=self.handler,
+                split_statements=self.split_statements,
+                return_last=self.return_last,
+            )
+        else:
+            output = hook.run(
+                sql=self.sql,
+                autocommit=self.autocommit,
+                parameters=self.parameters,
+                split_statements=self.split_statements,
+            )
+
+        if hasattr(self, "_process_output"):
+            for out in output:
+                self._process_output(*out)
+
+        return output
+
+    def prepare_template(self) -> None:
+        """Parse template file for attribute parameters."""
+        if isinstance(self.parameters, str):
+            self.parameters = ast.literal_eval(self.parameters)
+
 
 class SQLColumnCheckOperator(BaseSQLOperator):
     """
@@ -171,6 +238,7 @@ class SQLColumnCheckOperator(BaseSQLOperator):
     - geq_to: value that results should be greater than or equal to
     - leq_to: value that results should be less than or equal to
     - tolerance: the percentage that the result may be off from the expected value
+    - partition_clause: an extra clause passed into a WHERE statement to partition data
 
     :param table: the table to run checks on
     :param column_mapping: the dictionary of columns and their associated checks, e.g.
@@ -181,6 +249,7 @@ class SQLColumnCheckOperator(BaseSQLOperator):
             "col_name": {
                 "null_check": {
                     "equal_to": 0,
+                    "partition_clause": "foreign_key IS NOT NULL",
                 },
                 "min": {
                     "greater_than": 5,
@@ -200,6 +269,8 @@ class SQLColumnCheckOperator(BaseSQLOperator):
 
     :param conn_id: the connection ID used to connect to the database
     :param database: name of database which overwrite the defined one in connection
+    :param accept_none: whether or not to accept None values returned by the query. If true, converts None
+        to 0.
 
     .. seealso::
         For more information on how to use this operator, take a look at the guide:
@@ -208,69 +279,105 @@ class SQLColumnCheckOperator(BaseSQLOperator):
 
     template_fields = ("partition_clause",)
 
+    sql_check_template = """
+        SELECT '{column}' AS col_name, '{check}' AS check_type, {column}_{check} AS check_result
+        FROM (SELECT {check_statement} AS {column}_{check} FROM {table} {partition_clause}) AS sq
+    """
+
     column_checks = {
-        "null_check": "SUM(CASE WHEN column IS NULL THEN 1 ELSE 0 END) AS column_null_check",
-        "distinct_check": "COUNT(DISTINCT(column)) AS column_distinct_check",
-        "unique_check": "COUNT(column) - COUNT(DISTINCT(column)) AS column_unique_check",
-        "min": "MIN(column) AS column_min",
-        "max": "MAX(column) AS column_max",
+        "null_check": "SUM(CASE WHEN {column} IS NULL THEN 1 ELSE 0 END)",
+        "distinct_check": "COUNT(DISTINCT({column}))",
+        "unique_check": "COUNT({column}) - COUNT(DISTINCT({column}))",
+        "min": "MIN({column})",
+        "max": "MAX({column})",
     }
 
     def __init__(
         self,
         *,
         table: str,
-        column_mapping: Dict[str, Dict[str, Any]],
-        partition_clause: Optional[str] = None,
-        conn_id: Optional[str] = None,
-        database: Optional[str] = None,
+        column_mapping: dict[str, dict[str, Any]],
+        partition_clause: str | None = None,
+        conn_id: str | None = None,
+        database: str | None = None,
+        accept_none: bool = True,
         **kwargs,
     ):
         super().__init__(conn_id=conn_id, database=database, **kwargs)
-        for checks in column_mapping.values():
-            for check, check_values in checks.items():
-                self._column_mapping_validation(check, check_values)
 
         self.table = table
         self.column_mapping = column_mapping
         self.partition_clause = partition_clause
-        # OpenLineage needs a valid SQL query with the input/output table(s) to parse
-        self.sql = f"SELECT * FROM {self.table};"
+        self.accept_none = accept_none
 
-    def execute(self, context: 'Context'):
+        def _build_checks_sql():
+            for column, checks in self.column_mapping.items():
+                for check, check_values in checks.items():
+                    self._column_mapping_validation(check, check_values)
+                yield self._generate_sql_query(column, checks)
+
+        checks_sql = "UNION ALL".join(_build_checks_sql())
+
+        self.sql = f"SELECT col_name, check_type, check_result FROM ({checks_sql}) AS check_columns"
+
+    def execute(self, context: Context):
         hook = self.get_db_hook()
-        failed_tests = []
-        for column in self.column_mapping:
-            checks = [*self.column_mapping[column]]
-            checks_sql = ",".join([self.column_checks[check].replace("column", column) for check in checks])
-            partition_clause_statement = f"WHERE {self.partition_clause}" if self.partition_clause else ""
-            self.sql = f"SELECT {checks_sql} FROM {self.table} {partition_clause_statement};"
-            records = hook.get_first(self.sql)
+        records = hook.get_records(self.sql)
 
-            if not records:
-                raise AirflowException(f"The following query returned zero rows: {self.sql}")
+        if not records:
+            self._raise_exception(f"The following query returned zero rows: {self.sql}")
 
-            self.log.info("Record: %s", records)
+        self.log.info("Record: %s", records)
 
-            for idx, result in enumerate(records):
-                tolerance = self.column_mapping[column][checks[idx]].get("tolerance")
+        for column, check, result in records:
+            tolerance = self.column_mapping[column][check].get("tolerance")
 
-                self.column_mapping[column][checks[idx]]["result"] = result
-                self.column_mapping[column][checks[idx]]["success"] = self._get_match(
-                    self.column_mapping[column][checks[idx]], result, tolerance
-                )
-
-            failed_tests.extend(_get_failed_checks(self.column_mapping[column], column))
-        if failed_tests:
-            raise AirflowException(
-                f"Test failed.\nResults:\n{records!s}\n"
-                "The following tests have failed:"
-                f"\n{''.join(failed_tests)}"
+            self.column_mapping[column][check]["result"] = result
+            self.column_mapping[column][check]["success"] = self._get_match(
+                self.column_mapping[column][check], result, tolerance
             )
 
+        failed_tests = [
+            f"Column: {col}\n\tCheck: {check},\n\tCheck Values: {check_values}\n"
+            for col, checks in self.column_mapping.items()
+            for check, check_values in checks.items()
+            if not check_values["success"]
+        ]
+        if failed_tests:
+            exception_string = (
+                f"Test failed.\nResults:\n{records!s}\n"
+                f"The following tests have failed:\n{''.join(failed_tests)}"
+            )
+            self._raise_exception(exception_string)
+
         self.log.info("All tests have passed")
 
+    def _generate_sql_query(self, column, checks):
+        def _generate_partition_clause(check):
+            if self.partition_clause and "partition_clause" not in checks[check]:
+                return f"WHERE {self.partition_clause}"
+            elif not self.partition_clause and "partition_clause" in checks[check]:
+                return f"WHERE {checks[check]['partition_clause']}"
+            elif self.partition_clause and "partition_clause" in checks[check]:
+                return f"WHERE {self.partition_clause} AND {checks[check]['partition_clause']}"
+            else:
+                return ""
+
+        checks_sql = "UNION ALL".join(
+            self.sql_check_template.format(
+                check_statement=self.column_checks[check].format(column=column),
+                check=check,
+                table=self.table,
+                column=column,
+                partition_clause=_generate_partition_clause(check),
+            )
+            for check in checks
+        )
+        return checks_sql
+
     def _get_match(self, check_values, record, tolerance=None) -> bool:
+        if record is None and self.accept_none:
+            record = 0
         match_boolean = True
         if "geq_to" in check_values:
             if tolerance is not None:
@@ -369,13 +476,15 @@ class SQLTableCheckOperator(BaseSQLOperator):
     Checks should be written to return a boolean result.
 
     :param table: the table to run checks on
-    :param checks: the dictionary of checks, e.g.:
+    :param checks: the dictionary of checks, where check names are followed by a dictionary containing at
+        least a check statement, and optionally a partition clause, e.g.:
 
     .. code-block:: python
 
         {
             "row_count_check": {"check_statement": "COUNT(*) = 1000"},
             "column_sum_check": {"check_statement": "col_a + col_b < col_c"},
+            "third_check": {"check_statement": "MIN(col) = 1", "partition_clause": "col IS NOT NULL"},
         }
 
 
@@ -397,18 +506,19 @@ class SQLTableCheckOperator(BaseSQLOperator):
     template_fields = ("partition_clause",)
 
     sql_check_template = """
-        SELECT '_check_name' AS check_name, MIN(_check_name) AS check_result
-        FROM (SELECT CASE WHEN check_statement THEN 1 ELSE 0 END AS _check_name FROM table) AS sq
+    SELECT '{check_name}' AS check_name, MIN({check_name}) AS check_result
+    FROM (SELECT CASE WHEN {check_statement} THEN 1 ELSE 0 END AS {check_name}
+          FROM {table} {partition_clause}) AS sq
     """
 
     def __init__(
         self,
         *,
         table: str,
-        checks: Dict[str, Dict[str, Any]],
-        partition_clause: Optional[str] = None,
-        conn_id: Optional[str] = None,
-        database: Optional[str] = None,
+        checks: dict[str, dict[str, Any]],
+        partition_clause: str | None = None,
+        conn_id: str | None = None,
+        database: str | None = None,
         **kwargs,
     ):
         super().__init__(conn_id=conn_id, database=database, **kwargs)
@@ -416,46 +526,56 @@ class SQLTableCheckOperator(BaseSQLOperator):
         self.table = table
         self.checks = checks
         self.partition_clause = partition_clause
-        # OpenLineage needs a valid SQL query with the input/output table(s) to parse
-        self.sql = f"SELECT * FROM {self.table};"
+        self.sql = f"SELECT check_name, check_result FROM ({self._generate_sql_query()}) AS check_table"
 
-    def execute(self, context: 'Context'):
+    def execute(self, context: Context):
         hook = self.get_db_hook()
-        checks_sql = " UNION ALL ".join(
-            [
-                self.sql_check_template.replace("check_statement", value["check_statement"])
-                .replace("_check_name", check_name)
-                .replace("table", self.table)
-                for check_name, value in self.checks.items()
-            ]
-        )
-        partition_clause_statement = f"WHERE {self.partition_clause}" if self.partition_clause else ""
-        self.sql = f"""
-            SELECT check_name, check_result FROM ({checks_sql})
-            AS check_table {partition_clause_statement}
-        """
-
         records = hook.get_records(self.sql)
 
         if not records:
-            raise AirflowException(f"The following query returned zero rows: {self.sql}")
+            self._raise_exception(f"The following query returned zero rows: {self.sql}")
 
         self.log.info("Record:\n%s", records)
 
         for row in records:
             check, result = row
-            self.checks[check]["success"] = parse_boolean(str(result))
+            self.checks[check]["success"] = _parse_boolean(str(result))
 
-        failed_tests = _get_failed_checks(self.checks)
+        failed_tests = [
+            f"\tCheck: {check},\n\tCheck Values: {check_values}\n"
+            for check, check_values in self.checks.items()
+            if not check_values["success"]
+        ]
         if failed_tests:
-            raise AirflowException(
+            exception_string = (
                 f"Test failed.\nQuery:\n{self.sql}\nResults:\n{records!s}\n"
-                "The following tests have failed:"
-                f"\n{', '.join(failed_tests)}"
+                f"The following tests have failed:\n{', '.join(failed_tests)}"
             )
+            self._raise_exception(exception_string)
 
         self.log.info("All tests have passed")
 
+    def _generate_sql_query(self):
+        def _generate_partition_clause(check_name):
+            if self.partition_clause and "partition_clause" not in self.checks[check_name]:
+                return f"WHERE {self.partition_clause}"
+            elif not self.partition_clause and "partition_clause" in self.checks[check_name]:
+                return f"WHERE {self.checks[check_name]['partition_clause']}"
+            elif self.partition_clause and "partition_clause" in self.checks[check_name]:
+                return f"WHERE {self.partition_clause} AND {self.checks[check_name]['partition_clause']}"
+            else:
+                return ""
+
+        return "UNION ALL".join(
+            self.sql_check_template.format(
+                check_statement=value["check_statement"],
+                check_name=check_name,
+                table=self.table,
+                partition_clause=_generate_partition_clause(check_name),
+            )
+            for check_name, value in self.checks.items()
+        )
+
 
 class SQLCheckOperator(BaseSQLOperator):
     """
@@ -488,6 +608,7 @@ class SQLCheckOperator(BaseSQLOperator):
     :param sql: the sql to be executed. (templated)
     :param conn_id: the connection ID used to connect to the database.
     :param database: name of database which overwrite the defined one in connection
+    :param parameters: (optional) the parameters to render the SQL query with.
     """
 
     template_fields: Sequence[str] = ("sql",)
@@ -499,20 +620,27 @@ class SQLCheckOperator(BaseSQLOperator):
     ui_color = "#fff7e6"
 
     def __init__(
-        self, *, sql: str, conn_id: Optional[str] = None, database: Optional[str] = None, **kwargs
+        self,
+        *,
+        sql: str,
+        conn_id: str | None = None,
+        database: str | None = None,
+        parameters: Iterable | Mapping | None = None,
+        **kwargs,
     ) -> None:
         super().__init__(conn_id=conn_id, database=database, **kwargs)
         self.sql = sql
+        self.parameters = parameters
 
-    def execute(self, context: 'Context'):
+    def execute(self, context: Context):
         self.log.info("Executing SQL check: %s", self.sql)
-        records = self.get_db_hook().get_first(self.sql)
+        records = self.get_db_hook().get_first(self.sql, self.parameters)
 
         self.log.info("Record: %s", records)
         if not records:
-            raise AirflowException("The query returned None")
+            self._raise_exception(f"The following query returned zero rows: {self.sql}")
         elif not all(bool(r) for r in records):
-            raise AirflowException(f"Test failed.\nQuery:\n{self.sql}\nResults:\n{records!s}")
+            self._raise_exception(f"Test failed.\nQuery:\n{self.sql}\nResults:\n{records!s}")
 
         self.log.info("Success.")
 
@@ -544,8 +672,8 @@ class SQLValueCheckOperator(BaseSQLOperator):
         sql: str,
         pass_value: Any,
         tolerance: Any = None,
-        conn_id: Optional[str] = None,
-        database: Optional[str] = None,
+        conn_id: str | None = None,
+        database: str | None = None,
         **kwargs,
     ):
         super().__init__(conn_id=conn_id, database=database, **kwargs)
@@ -555,12 +683,12 @@ class SQLValueCheckOperator(BaseSQLOperator):
         self.tol = tol if isinstance(tol, float) else None
         self.has_tolerance = self.tol is not None
 
-    def execute(self, context: 'Context'):
+    def execute(self, context: Context):
         self.log.info("Executing SQL check: %s", self.sql)
         records = self.get_db_hook().get_first(self.sql)
 
         if not records:
-            raise AirflowException("The query returned None")
+            self._raise_exception(f"The following query returned zero rows: {self.sql}")
 
         pass_value_conv = _convert_to_float_if_possible(self.pass_value)
         is_numeric_value_check = isinstance(pass_value_conv, float)
@@ -589,7 +717,7 @@ class SQLValueCheckOperator(BaseSQLOperator):
             tests = []
 
         if not all(tests):
-            raise AirflowException(error_msg)
+            self._raise_exception(error_msg)
 
     def _to_float(self, records):
         return [float(record) for record in records]
@@ -648,20 +776,20 @@ class SQLIntervalCheckOperator(BaseSQLOperator):
         self,
         *,
         table: str,
-        metrics_thresholds: Dict[str, int],
-        date_filter_column: Optional[str] = "ds",
+        metrics_thresholds: dict[str, int],
+        date_filter_column: str | None = "ds",
         days_back: SupportsAbs[int] = -7,
-        ratio_formula: Optional[str] = "max_over_min",
+        ratio_formula: str | None = "max_over_min",
         ignore_zero: bool = True,
-        conn_id: Optional[str] = None,
-        database: Optional[str] = None,
+        conn_id: str | None = None,
+        database: str | None = None,
         **kwargs,
     ):
         super().__init__(conn_id=conn_id, database=database, **kwargs)
         if ratio_formula not in self.ratio_formulas:
             msg_template = "Invalid diff_method: {diff_method}. Supported diff methods are: {diff_methods}"
 
-            raise AirflowException(
+            raise AirflowFailException(
                 msg_template.format(diff_method=ratio_formula, diff_methods=self.ratio_formulas)
             )
         self.ratio_formula = ratio_formula
@@ -677,7 +805,7 @@ class SQLIntervalCheckOperator(BaseSQLOperator):
         self.sql1 = sqlt + "'{{ ds }}'"
         self.sql2 = sqlt + "'{{ macros.ds_add(ds, " + str(self.days_back) + ") }}'"
 
-    def execute(self, context: 'Context'):
+    def execute(self, context: Context):
         hook = self.get_db_hook()
         self.log.info("Using ratio formula: %s", self.ratio_formula)
         self.log.info("Executing SQL check: %s", self.sql2)
@@ -686,14 +814,14 @@ class SQLIntervalCheckOperator(BaseSQLOperator):
         row1 = hook.get_first(self.sql1)
 
         if not row2:
-            raise AirflowException(f"The query {self.sql2} returned None")
+            self._raise_exception(f"The following query returned zero rows: {self.sql2}")
         if not row1:
-            raise AirflowException(f"The query {self.sql1} returned None")
+            self._raise_exception(f"The following query returned zero rows: {self.sql1}")
 
         current = dict(zip(self.metrics_sorted, row1))
         reference = dict(zip(self.metrics_sorted, row2))
 
-        ratios: Dict[str, Optional[int]] = {}
+        ratios: dict[str, int | None] = {}
         test_results = {}
 
         for metric in self.metrics_sorted:
@@ -741,7 +869,7 @@ class SQLIntervalCheckOperator(BaseSQLOperator):
                     ratios[k],
                     self.metrics_thresholds[k],
                 )
-            raise AirflowException(f"The following tests have failed:\n {', '.join(sorted(failed_tests))}")
+            self._raise_exception(f"The following tests have failed:\n {', '.join(sorted(failed_tests))}")
 
         self.log.info("All tests have passed")
 
@@ -772,8 +900,8 @@ class SQLThresholdCheckOperator(BaseSQLOperator):
         sql: str,
         min_threshold: Any,
         max_threshold: Any,
-        conn_id: Optional[str] = None,
-        database: Optional[str] = None,
+        conn_id: str | None = None,
+        database: str | None = None,
         **kwargs,
     ):
         super().__init__(conn_id=conn_id, database=database, **kwargs)
@@ -781,9 +909,11 @@ class SQLThresholdCheckOperator(BaseSQLOperator):
         self.min_threshold = _convert_to_float_if_possible(min_threshold)
         self.max_threshold = _convert_to_float_if_possible(max_threshold)
 
-    def execute(self, context: 'Context'):
+    def execute(self, context: Context):
         hook = self.get_db_hook()
         result = hook.get_first(self.sql)[0]
+        if not result:
+            self._raise_exception(f"The following query returned zero rows: {self.sql}")
 
         if isinstance(self.min_threshold, float):
             lower_bound = self.min_threshold
@@ -815,10 +945,10 @@ class SQLThresholdCheckOperator(BaseSQLOperator):
                 f'DAG: {self.dag_id}\nTask_id: {meta_data.get("task_id")}\n'
                 f'Check description: {meta_data.get("description")}\n'
                 f"SQL: {self.sql}\n"
-                f'Result: {result} is not within thresholds '
+                f"Result: {result} is not within thresholds "
                 f'{meta_data.get("min_threshold")} and {meta_data.get("max_threshold")}'
             )
-            raise AirflowException(error_msg)
+            self._raise_exception(error_msg)
 
         self.log.info("Test %s Successful.", self.task_id)
 
@@ -856,11 +986,11 @@ class BranchSQLOperator(BaseSQLOperator, SkipMixin):
         self,
         *,
         sql: str,
-        follow_task_ids_if_true: List[str],
-        follow_task_ids_if_false: List[str],
+        follow_task_ids_if_true: list[str],
+        follow_task_ids_if_false: list[str],
         conn_id: str = "default_conn_id",
-        database: Optional[str] = None,
-        parameters: Optional[Union[Iterable, Mapping]] = None,
+        database: str | None = None,
+        parameters: Iterable | Mapping | None = None,
         **kwargs,
     ) -> None:
         super().__init__(conn_id=conn_id, database=database, **kwargs)
@@ -869,7 +999,7 @@ class BranchSQLOperator(BaseSQLOperator, SkipMixin):
         self.follow_task_ids_if_true = follow_task_ids_if_true
         self.follow_task_ids_if_false = follow_task_ids_if_false
 
-    def execute(self, context: 'Context'):
+    def execute(self, context: Context):
         self.log.info(
             "Executing: %s (with parameters %s) with connection: %s",
             self.sql,
@@ -901,7 +1031,7 @@ class BranchSQLOperator(BaseSQLOperator, SkipMixin):
                     follow_branch = self.follow_task_ids_if_true
             elif isinstance(query_result, str):
                 # return result is not Boolean, try to convert from String to Boolean
-                if parse_boolean(query_result):
+                if _parse_boolean(query_result):
                     follow_branch = self.follow_task_ids_if_true
             elif isinstance(query_result, int):
                 if bool(query_result):
@@ -919,17 +1049,3 @@ class BranchSQLOperator(BaseSQLOperator, SkipMixin):
             )
 
         self.skip_all_except(context["ti"], follow_branch)
-
-
-def _convert_to_float_if_possible(s):
-    """
-    A small helper function to convert a string to a numeric value
-    if appropriate
-
-    :param s: the string to be converted
-    """
-    try:
-        ret = float(s)
-    except (ValueError, TypeError):
-        ret = s
-    return ret
